//
// Pollin Funk-AVR-Evaluationsboard v1.1
// PFA_uart.c
// v 1.0
//

#include "PFA_uart.h"

/*
    Atmega8 
    (USART1 auf AT90USB1287 siehe Kommentare)

    Externer Quarz-Oszillator: 12 MHz (USART1 auf AT90USB1287 8 MHz)
    Optimierung: -Os

    RS232-Einstellungen:
    9600 Baud, 8 Bits, No Parity, 1 Stopbit
*/
void UART_init(void)
{
    // Baudrate setzen
    // 9600 Baud bei F_CPU 12 MHz => Baudratenfehler = +0,2%
    // 9600 Baud bei F_CPU 8 MHz  => Baudratenfehler = +0,2%
    UBRRH = (uint8_t) ((F_CPU / (16 * 9600L) - 1) >> 8);		// AT90USB1287: UBRRH1 = ...
    UBRRL = (uint8_t)  (F_CPU / (16 * 9600L) - 1);					// AT90USB1287: UBRRL1 = ...

    //      Empfangen,   Senden erlauben
    UCSRB = (1<<RXEN) | (1<<TXEN);													
    // AT90USB1287: UCSR1B = (1<<RXEN1) | (1<<TXEN1);

    // Frame Format:              8 Bits,                  No Parity,          1 Stopbit
    UCSRC = (1<<URSEL) | ((1<<UCSZ1) | (1<<UCSZ0)) | ((0<<UPM1) | (0<<UPM0)) | (0<<USBS);
    // AT90USB1287: UCSR1C = ((1<<UCSZ11) | (1<<UCSZ10));
}


// Auf Zeichen im UART Eingang prfen
uint8_t UART_peekchar(void)
{
    // sofort zurckkehren und Zustand melden
    return UCSRA & (1<<RXC);
    // AT90USB1287: return UCSR1A & (1<<RXC1);
}


// Auf Zeichen im UART Eingang warten
uint8_t UART_getchar(void)
{
    // Warte bis UART Eingang voll
    while ( !(UCSRA & (1<<RXC)) )
    // AT90USB1287: while ( !(UCSR1A & (1<<RXC1)) )
        ;
    return UDR;
    // AT90USB1287: return UDR1;
}


// Zeichen in UART Ausgang geben
void UART_putchar(char z)
{
    // Warte bis UART Ausgang frei
    while ( !(UCSRA & (1<<UDRE)) )
    // AT90USB1287: while ( !(UCSR1A & (1<<UDRE1)) )
        ;
    UDR = z;
    // AT90USB1287: UDR1 = z;
}


// Zeichenkette (String) in UART Ausgang geben
void UART_puts(char *s)
{
    while ( *s )
    {
        UART_putchar(*s);
        s++;
    }
}

// EOF PFA_uart.c