// Testboard: Pollin Funk-AVR-Evaluationsboard v1.1, 12 MHz, ATmega 8)
//            Externer Taster PC0 und PC2
// Auskommentieren bei AT90USBKEY !
// #define SBTEST

// DEBUG
// 0: kein DEBUG
//
// Bitmaske:
// 1: DEBUG "Tasten" aktiv
// 2: DEBUG "UART" aktiv
// 4: DEBUG "Puls" aktiv
#define DEBUG 1 // +2+4

#ifndef F_CPU
#define F_CPU 8000000UL     /* Quarz mit 8 Mhz  */
#endif
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

typedef unsigned char  u8;
typedef unsigned short u16;

#ifdef SBTEST
#define TCCR0B  TCCR0
#define TIMSK0  TIMSK
#define XTAL    F_CPU
#else
#define XTAL  8e6    // 8MHz
#endif

#define KEY_PIN   PINC     // C1..0 = inputs
#define KEY_PORT  PORTC
#define KEY_DDR   DDRC
#define KEY_COL0  PORTC0
#define KEY_COL1  PORTC1
#define KEY_ROW0  PORTC2   // C6..2 = outputs
#define KEY_ROW1  PORTC3
#define KEY_ROW2  PORTC4
#define KEY_ROW3  PORTC5
#define KEY_ROW4  PORTC6
#define ROWMASKE \
((1<<KEY_ROW0)|(1<<KEY_ROW1)|(1<<KEY_ROW2)|(1<<KEY_ROW3)|(1<<KEY_ROW4))

#if (DEBUG & 2)
#include <stdlib.h>
#include "PFA_uart.h"

void debug_u16(char *s, u16 w)
{
  static u8 init_done = 0;
  char p[8 * sizeof (unsigned int) + 1];

  if ( !init_done )
  {
    UART_init();    // 9600 Baud 8-N-1
    init_done = 1;
  }

  if (s)
  {
    UART_puts(s);
    UART_puts(" => ");
  }

  utoa(w, p, 2);
  UART_puts(p);
  UART_puts("\n\r");
}
#endif /* (DEBUG & 2) */

static inline
u16 key_scan( void )
{
  u16 keys = 0;

  // row0..4 = pullup on
  // col0..1 LOW wenn Ausgabe aktiv
  KEY_PORT = ROWMASKE;

  // Die Tasten + = T1 bis FN = T5
  KEY_DDR = 1<<KEY_COL0;        // col0 = low
  _delay_us(10);
  // setzen Bits 1..5 in keys
  keys = ((u16) (KEY_PIN & ROWMASKE)) >> 1;

  // Tasten F6 = T6 bis F1 = T10
  KEY_DDR = 1<<KEY_COL1;        // col1 = low
  _delay_us(10);
  // setzen Bits 6..10 in keys
  keys ^= ((u16) (KEY_PIN & ROWMASKE)) << 4;

#if (DEBUG & 1)
#ifdef SBTEST
  // PC6 ist beim ATmega8 RESET
  // aus Ergebnis lschen
  keys |= ((1<<5) | (1<<10));
#endif /* SBTEST */

  //DIAGNOSE
  if ( ((~keys) & 0x07FE) )
    PORTD |= (1<<PD6);   // ja => D5 Grn AN
  else
    PORTD &= ~(1<<PD6);  // D5 Grn AUS
#endif /* (DEBUG & 1) */

#if (DEBUG & 2)
  // Nur im Extremfall anschalten - braucht viel Rechenzeit,
  // so dass das Blinken in main nicht mehr funktioniert!
  // debug_u16("~keys", ~keys);
#endif /* (DEBUG & 2) */

  return ~keys;
}

u16 key_state;        // debounced key state:
                      // bit = 1: key pressed
u16 key_press;        // key press detect


ISR(TIMER0_OVF_vect)
{
  static u16 ct0, ct1;
  u16 i;

  TCNT0 = (u16)(256 - XTAL / 1024 / 100);  // 100Hz (10ms)

  i = key_state ^ key_scan();   // key changed ?
  ct0 = ~( ct0 & i );           // reset or count ct0
  ct1 = ct0 ^ (ct1 & i);        // reset or count ct1
  i &= ct0 & ct1;               // count until roll over ?
  key_state ^= i;               // then toggle debounced state
  key_press |= key_state & i;   // 0->1: key press detect
}


u16 get_key_press(u16 key_mask)
{
#if (DEBUG & 2)
  debug_u16("key_state", key_state);
  debug_u16("key_press", key_press);
#endif /* (DEBUG & 2) */

  cli();
  key_mask &= key_press;    // read key(s)
  key_press ^= key_mask;    // clear key(s)
  sei();

#if (DEBUG & 2)
  debug_u16("key_mask ", key_mask);
#endif /* (DEBUG & 2) */

  return key_mask;
}


int main(void)
{
  u16 taste;

#if (DEBUG)
  // DIAGNOSE
  DDRD |= (1<<PD5) | (1<<PD6);
#endif /* (DEBUG > 0) */

  TCCR0B = (1<<CS02)|(1<<CS00);        // XTAL / 1024
  TIMSK0 = 1<<TOIE0;

  sei();

#if (DEBUG & 4)
  // DIAGNOSE
  // Puls fhlen: 10 s lang mit 1 Hz Blinken
  {
     int i = 20;           // 10 AN Phasen + 10 AUS Phasen zu je 500 ms
     PORTD |= (1<<PD5);    // AN
     do
     {
        _delay_ms(500);
        PORTD ^= (1<<PD5); // TOGGLE
        i -= 1;
     }
     while (i);
     PORTD &= ~(1<<PD5);   // AUS
  }
#endif /* (DEBUG & 4) */

  for(;;)
  {
    if( get_key_press( 1<<1 ) )    // "Taste 1" merken
      taste ^= (1<<1);

#if (DEBUG)
    // DIAGNOSE
    if ( (taste & (1<<1)) )
    {
      PORTD ^= (1<<PD5);   // 0.5s Blinken
      _delay_ms(500);
    }
    else
    {
      PORTD ^= (1<<PD5);   // 2s Blinken
      _delay_ms(2000);
    }
#endif /* (DEBUG) */
  }
}
